/*
 * Copyright 2002-2005 webapp2.net
 *
 * 본 파일은 the Apache License, Version 2.0를 따릅니다:
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 */
package net.webapp2.forum.dao;

import java.util.Date;
import java.util.List;

import net.webapp2.forum.domain.Category;
import net.webapp2.forum.domain.Forum;
import net.webapp2.forum.domain.Post;
import net.webapp2.member.domain.Member;

import org.springframework.test.AbstractTransactionalDataSourceSpringContextTests;

/**
 * PostDao 객체 테스트케이스
 *
 * @author Ahn, Younghoe
 * @author Baik, Keesun
 */
public class PostDaoTest extends
		AbstractTransactionalDataSourceSpringContextTests {

	private PostDao postDao;

	/**
	 * 부모 Post가 없는 Post 객체 저장 테스트
	 */
	public void testSaveWithoutParent() {

		// Member 객체 생성
		String id = "springmembr";
		String alias = "younghoe";
		Member member = new Member();
		member.setId(id);
		member.setAlias(alias);

		// Post 객체 생성
		Post post = new Post();
		String title = "testSaveWithoutParent";
		String contents = "다들 난리인데.. 난 모르겠삼";

		post.setAuthor(member);
		post.setTitle(title);
		post.setContents(contents);
		post.setForum(new Forum(new Category(1)));
		post.setCreated(new Date());

		// do it!
		postDao.save(post);

		// assertion
		assertNotNull(post.getId());

	}

	/**
	 * 부모 Post가 있는 Post 객체 저장 테스트
	 */
	public void testSaveWithParent() {

		// 로그인 이후
		String id = "springmembr";
		String alias = "younghoe";

		Member member = new Member();
		member.setId(id);
		member.setAlias(alias);

		Post post = new Post();
		String title = "testSaveWithParent";
		String contents = "다들 난리인데.. 난 모르겠삼";

		post.setAuthor(member);
		post.setTitle(title);
		post.setContents(contents);
		post.setForum(new Forum(new Category(1)));
		post.setCreated(new Date());

		Post parent = postDao.load(1);
		post.setParent(parent);

		Integer beforeUpdate = parent.getCountOfReply();

		parent.setCountOfReply(beforeUpdate+1);
		postDao.update(parent);

		assertEquals(new Integer(beforeUpdate+1), parent.getCountOfReply());


		postDao.save(post);

		// load for assertion
		Post saved = postDao.load(post.getId());

		assertNotNull(saved);
		assertEquals(parent.getId(), saved.getParent().getId());

	}

	/**
	 * 부모 Post가 없는 Post 객체 로딩 테스트
	 */
	public void testLoadWithoutParent() {

		// 사용자 선택
		Integer id = 1;

		Post starter = postDao.load(id);

		assertNotNull(starter.getAuthor());
		assertNotNull(starter.getContents());
		assertNotNull(starter.getCreated());
		assertNotNull(starter.getForum());
		assertNotNull(starter.getTitle());

		assertNull(starter.getParent());
	}


	/**
	 * 부모 Post가 있는 Post 객체 로딩 테스트
	 */
	public void testLoadWithParent(){

		//사용자 선택
		Integer id = 2;

		Post reply = postDao.loadReply(id);
		Post starter = reply.getParent();

		assertNotNull(starter.getAuthor());
		assertNotNull(starter.getContents());
		assertNotNull(starter.getCreated());
		assertNotNull(starter.getForum());
		assertNotNull(starter.getTitle());

		assertNull(starter.getParent());
	}

	/**
	 * Post 수정하기 테스트
	 */
	public void testUpdatePost(){

		//load.
		Post post = postDao.load(3);

		assertNotNull(post);

		//modify.
		String title = "dao test에서 바꾸기";
		String contents = "내 여자로 만들고 싶어~";
		post.setTitle(title);
		post.setContents(contents);
		post.setUpdated(new Date());
		//update.
		postDao.update(post);

		//reload.
		Post updated = postDao.load(3);

		assertNotNull(updated);
		assertEquals(title, updated.getTitle());
		assertEquals(contents, updated.getContents());
		assertNotNull(updated.getUpdated());

	}

	/**
	 * Post 삭제하기 테스트
	 */
	public void testDeletePost(){

		//delete.
		postDao.delete(4);

		//load.
		Post post = postDao.load(4);

		//assertion.
		assertNull(post);

	}

	/**
	 * 원하는 갯수 만큼 Post 보기 테스트
	 */
	public void testList(){

		//load.
		List<Post> starters = postDao.list(2);

		//assertion.
		assertNotNull(starters);

		System.out.println("목록 가져오기");

		for( Post p : starters )
			System.out.println( "ID : " + p.getId() +
					" Member.Id : " + p.getAuthor().getId() +
					" Title : " + p.getTitle() +
					" Contents : " + p.getContents() );

		System.out.println();
	}

	/**
	 * 원하는 Post 찾기 테스트
	 */
	public void testFindPost(){

		//글 제목에 "첫번째"가 들어가는 post들 검색하기.
		List<Post> posts = postDao.find("title", "첫번째" );

		assertNotNull(posts);

		System.out.println("글 제목 검색하기");

		for( Post p : posts )
			System.out.println( "ID : " + p.getId() +
					" Member.Id : " + p.getAuthor().getId() +
					" Title : " + p.getTitle() +
					" Contents : " + p.getContents() );

		System.out.println();

		//글 내용에 "처음"가 들어가는 post들 검색하기.
		posts = postDao.find("contents", "답글");

		assertNotNull(posts);

		System.out.println("글 내용 검색하기");

		for( Post p : posts )
			System.out.println( "ID : " + p.getId() +
					" Member.Id : " + p.getAuthor().getId() +
					" Title : " + p.getTitle() +
					" Contents : " + p.getContents() );

		System.out.println();

		//글쓴이 이름으로 post들 검색하기.(특정 키워드 들어가는 사용자이름 찾기.)
		posts = postDao.find("fk_member_id", "young" );

		assertNotNull(posts);

		System.out.println("글쓴이 검색하기");

		for( Post p : posts )
			System.out.println( "ID : " + p.getId() +
					" Member.Id : " + p.getAuthor().getId() +
					" Title : " + p.getTitle() +
					" Contents : " + p.getContents() );

		System.out.println();

	}

	@Override
	protected String[] getConfigLocations() {

		return new String[] { "file:conf/daoContext.xml",
				"file:conf/daoContext-forum.xml" };

	}

	public void setPostDao(PostDao postDao) {
		this.postDao = postDao;
	}

}
