/*
 * Copyright 2002-2005 webapp2.net
 *
 * 본 파일은 the Apache License, Version 2.0를 따릅니다:
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 */
package net.webapp2.forum.service;

import java.util.List;

import net.webapp2.forum.domain.Category;
import net.webapp2.forum.domain.Forum;
import net.webapp2.forum.domain.Post;
import net.webapp2.member.domain.Member;

import org.springframework.test.AbstractDependencyInjectionSpringContextTests;

/**
 * @author Ahn, Younghoe
 * @author Baik, Keesun
 */
public class ForumServiceTest extends
		AbstractDependencyInjectionSpringContextTests {

	private ForumService forumService;

	/**
	 * 포럼에 의견 제기(첫글 쓰기)
	 */
	public void testPostStarter() {

		// 사전 조건: 로그인 선행
		String id = "springmembr";
		String alias = "younghoe";
		Member member = new Member();
		member.setId(id);
		member.setAlias(alias);

		// 특정 포럼으로 이동
		Category category = new Category(1);
		Forum forum = forumService.getForum(category);

		assertNotNull(forum);

		Post post = new Post();
		// 사용자 입력값
		String title = "Starter 글쓰기 Test";
		String contents = "다들 난리인데.. 난 모르겠삼";

		post.setAuthor(member);
		post.setTitle(title);
		post.setContents(contents);

		forumService.post(post, forum);

		assertNotNull(post.getCreated());
		assertEquals(category, post.getForum().getCategory());
		assertNotNull(post.getId());

	}

	/**
	 * FRM-002. 포럼에 답글 달기
	 */
	public void testReply() {

		//첫번째 포스트 ID는 1 (init-all-tables.sql 실행)
		Post starter = forumService.load(1);

		//로그인 이후
		String id = "springmembr";
		String alias = "younghoe";

		Member member = new Member();
		member.setId(id);
		member.setAlias(alias);

		//사용자 입력값
		String title = "Spring이 좋아요?";
		String contents = "다들 난리인데.. 난 모르겠삼";

		Post reply = new Post();
		reply.setAuthor(member);
		reply.setTitle(title);
		reply.setContents(contents);
		reply.setParent(starter);

		forumService.reply(reply);

		assertNotNull(reply.getCreated());
		assertNotNull(reply.getId());
		assertNotNull(reply.getForum());

	}

	/**
	 * FRM-003. 포럼에 글 보기
	 */
	public void testViewStarter(){

		Post starter = forumService.load(1);

		assertEquals(new Integer(1), starter.getId());
		assertNotNull(starter.getAuthor());
		assertNotNull(starter.getContents());
		assertNotNull(starter.getCreated());
		assertNotNull(starter.getForum());
		assertNotNull(starter.getTitle());

		assertNull(starter.getParent());
	}


	/**
	 * FRM-004. 포럼에 답글 보기
	 */
	public void testViewReply(){

		Post reply = forumService.loadReply(2);
		Post starter = reply.getParent();

		assertNotNull(starter);
		assertEquals(new Integer(1), starter.getId());
		assertNotNull(starter.getForum());
		assertNotNull(starter.getTitle());
		assertNotNull(starter.getAuthor());
		assertNotNull(starter.getContents());
		assertNotNull(starter.getCreated());
		assertNull(starter.getParent());
	}

	/**
	 * FRM-005. 포럼에 글 수정하기
	 */
	public void testUpdatePost(){

		//로그인 이후
		String id = "younghoe";
		Member member = new Member();
		member.setId(id);

		//글 불러오기
		Post post = forumService.load(3);

		//post 작성자와 member의 id를 비교합니다.
		Boolean canUpdate = forumService.checkMemberId(member, post);

		assertNotNull(canUpdate);
		assertEquals(new Boolean(true), canUpdate);

		//수정하기.
		String title = "수정했습니다.";
		String contents = "널 처음 본 순간 느꼈어";
		post.setTitle(title);
		post.setContents(contents);

		//update.
		forumService.update(post);

		//다시 불러오기.(수정된 post 불러오기.)
		Post updated = forumService.load(3);

		assertNotNull(updated);
		assertEquals(title, updated.getTitle());
		assertEquals(contents, updated.getContents());
		assertNotNull(updated.getUpdated());

	}

	/**
	 * FRM-006. 포럼에서 보고 있는 글 삭제하기.
	 */
	public void testDeleteStarter(){

		//로그인 이후
		String id = "younghoe";
		Member member = new Member();
		member.setId(id);

		//글 불러오기
		Post post = forumService.load(4);

		//post 작성자와 member의 id를 비교합니다.
		Boolean canUpdate = forumService.checkMemberId(member, post);

		assertNotNull(canUpdate);
		assertEquals(new Boolean(true), canUpdate);

		//post에 reply가 달려있는지 확인합니다.
		Boolean hasReply = forumService.hasReply(post);

		assertEquals(new Boolean(false), hasReply);

		//삭제하기.
		forumService.delete(post.getId());

		//확인하기.
		assertNull(forumService.load(4));
	}

	/**
	 * FRM-007. 포럼에서 답글 삭제하기.
	 */
	public void testDeleteReply(){

		//로그인 이후
		String id = "whiteship";
		Member member = new Member();
		member.setId(id);

		//글 불러오기
		Post post = forumService.load(5);

		//post 작성자와 member의 id를 비교합니다.
		Boolean canUpdate = forumService.checkMemberId(member, post);

		assertNotNull(canUpdate);
		assertEquals(new Boolean(true), canUpdate);

		//post에 parent가 있는지 확인합니다.
		Boolean hasParent = forumService.hasParent(post);

		assertEquals(new Boolean(true), hasParent);

		//삭제하기.
		forumService.deleteReply(post.getId());

		//확인하기.
		assertNull(forumService.load(5));
	}

	/**
	 * FRM-008. 포럼에서 원하는 수만큼 목록보기(starter 보기).
	 */
	public void testList(){

		//list의 매개변수로 원하는 starter 갯수를 지정해 줍니다.
		List<Post> starters = forumService.list(2);

		assertNotNull(starters);

		System.out.println("목록 가져오기");

		for( Post p : starters )
			System.out.println( "ID : " + p.getId() +
					" Member.Id : " + p.getAuthor().getId() +
					" Title : " + p.getTitle() +
					" Contents : " + p.getContents() );

		System.out.println();

	}

	/**
	 * FRM-009. 포럼에서 원하는 글 검색하기.
	 */
	public void testFindPost(){

		//글 제목에 "첫번째"가 들어가는 post들 검색하기.
		List<Post> posts = forumService.find("title", "첫번째" );

		assertNotNull(posts);

		System.out.println("글 제목 검색하기");

		for( Post p : posts )
			System.out.println( "ID : " + p.getId() +
					" Member.Id : " + p.getAuthor().getId() +
					" Title : " + p.getTitle() +
					" Contents : " + p.getContents() );

		System.out.println();

		//글 내용에 "처음"가 들어가는 post들 검색하기.
		posts = forumService.find("contents", "처음");

		assertNotNull(posts);

		System.out.println("글 내용 검색하기");

		for( Post p : posts )
			System.out.println( "ID : " + p.getId() +
					" Member.Id : " + p.getAuthor().getId() +
					" Title : " + p.getTitle() +
					" Contents : " + p.getContents() );

		System.out.println();

		//글쓴이 이름으로 post들 검색하기.(특정 키워드 들어가는 사용자이름 찾기.)
		posts = forumService.find("fk_member_id", "young" );

		assertNotNull(posts);

		System.out.println("글쓴이 검색하기");

		for( Post p : posts )
			System.out.println( "ID : " + p.getId() +
					" Member.Id : " + p.getAuthor().getId() +
					" Title : " + p.getTitle() +
					" Contents : " + p.getContents() );

		System.out.println();

	}

	@Override
	protected String[] getConfigLocations() {

		return new String[] {
				"file:conf/applicationContext-forum.xml",
				"file:conf/daoContext.xml",
				"file:conf/daoContext-forum.xml"
		};

	}

	public void setForumService(ForumService forumService) {
		this.forumService = forumService;
	}

}
