package net.webapp2.util.test;

import java.io.FileNotFoundException;
import java.io.IOException;

import javax.sql.DataSource;

import net.webapp2.util.jdbc.SimpleSqlExecutor;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.BeanFactory;
import org.testng.AssertJUnit;

/**
 * Spring 컨텍스트 파일 설정으로 IoC를 활용한 테스트를 수행
 *
 * @author younghoe
 *
 */
public abstract class TestBase {

	private SpringMock springMock;

	protected final Log logger = LogFactory.getLog(getClass());

	private BeanFactory factory;

	/**
	 * context.properties 파일의 위치를 변경하여 테스트하고자 할 때 매개변수로
	 * 파일 경로를 포함하는 properties 파일 위치를 지정.
	 * 생성자에서 작업을 수행하면 로그처리가 잘 되지 않으므로 @Configurtaion
	 * 이용을 권장함
	 *
	 * @param contextPropertiesFilePath
	 *            변경할 context.properties 파일의 위치
	 * @throws PropertyDefinitionNotFoundException
	 * @throws IOException
	 * @throws FileNotFoundException
	 */
	public TestBase(String contextPropertiesFilePath) throws FileNotFoundException, IOException, PropertyDefinitionNotFoundException {


		springMock = new SpringMock();
		springMock.setContextPropertiesFilePath(contextPropertiesFilePath);
		factory = springMock.loadContext();

	}

	/**
	 * 하위 클래스에서 생성자가 생기는 것을 막기 위해 try - catch 수행.
	 * 생성자에서 작업을 수행하면 로그처리가 잘 되지 않으므로 @Configurtaion
	 * 이용을 권장함
	 */
	public TestBase(){

		springMock = new SpringMock();
		try {
			factory = springMock.loadContext();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (PropertyDefinitionNotFoundException e) {
			e.printStackTrace();
		}

	}

	/**
	 * 테스트 과정에서 쓰이는 BeanFactory 반환
	 *
	 * @return 테스트 과정에서 쓰이는 BeanFactory 객체
	 */
	public BeanFactory getBeanFactory() {
		return this.factory;
	}

	/**
	 * 테스트용 Bean Factory가 관리하는 객체(bean) 반환
	 *
	 * @param beanName
	 *            객체 이름
	 * @return
	 */
	public Object getBean(String beanName) {
		return factory.getBean(beanName);
	}

	/**
	 * 예상되는 객체와 실제 결과 객체가 다른 경우 {@link java.lang.AssertionError AssertionError}를
	 * 발생함. 다른 메소드와의 일관성 유지를 위해 만듦
	 *
	 * @param expection
	 *            예상되는 객체
	 * @param actualResult
	 *            실제 결과 객체
	 * @see java.lang.AssertionError
	 */
	public void equals(Object expection, Object actualResult) {

		AssertJUnit.assertEquals(expection, actualResult);

	}

	/**
	 * 매개변수가 null인 경우 {@link java.lang.AssertionError AssertionError}를 발생함.
	 * TestNG의 {@link AssertJUnit#assertNotNull assertNotNull} 메소드의 경우에는
	 * {@link java.lang.AssertionError AssertionError} 발생시 출력문이 없어
	 * {@link AssertJUnit#assertEquals assertEquals} 스타일의 출력문을 대행하기 위해 작성
	 *
	 * @param object
	 *            null이 아닐 것으로 예상되는 객체
	 * @see java.lang.AssertionError
	 * @see AssertJUnit#assertNotNull
	 */
	public void notNull(Object object) {

		String errorMessage = "expected:<an Object> but was:<" + object + ">";
		AssertJUnit.assertNotNull(errorMessage, object);

	}

	/**
	 * 매개변수가 false가 아닌 경우 {@link java.lang.AssertionError AssertionError}를 발생함.
	 * TestNG의 {@link AssertJUnit#assertFalse assertFalse}메소드의 경우에는
	 * {@link java.lang.AssertionError AssertionError} 발생시 출력문이 없어
	 * {@link AssertJUnit#assertEquals assertEquals} 스타일의 출력문을 대행하기 위해 작성
	 *
	 * @param bool
	 *            false로 예상되는 변수
	 * @see java.lang.AssertionError
	 * @see org.testng.AssertJUnit#assertFalse
	 */
	public void isFalse(boolean bool) {

		AssertJUnit.assertEquals(false, bool);

	}

	/**
	 * 매개변수가 null이 아닌 경우 {@link java.lang.AssertionError AssertionError}를 발생함.
	 * TestNG의 {@link AssertJUnit#assertNull assertNull} 메소드의 경우에는
	 * {@link java.lang.AssertionError AssertionError} 발생시 출력문이 없어,
	 * {@link AssertJUnit#assertEquals assertEquals} 스타일의 출력문을 대행하기 위해 작성
	 *
	 * @param object
	 *            null로 예상되는 객체
	 * @see java.lang.AssertionError
	 * @see AssertJUnit#assertNull
	 */
	public void isNull(Object object) {

		AssertJUnit.assertEquals(null, object);

	}

	/**
	 * 매개변수가 true가 아닌 경우 {@link java.lang.AssertionError AssertionError}를 발생함.
	 * TestNG의 {@link AssertJUnit#assertTrue assertTrue} 메소드의 경우에는
	 * {@link java.lang.AssertionError AssertionError} 발생시 출력문이 없어
	 * {@link AssertJUnit#assertEquals assertEquals} 스타일의 출력문을 대행하기 위해 작성
	 *
	 * @param bool
	 *            true로 예상되는 변수
	 * @see java.lang.AssertionError
	 * @see AssertJUnit#assertTrue
	 */
	public void isTrue(boolean bool) {

		AssertJUnit.assertEquals(true, bool);

	}

	/**
	 * 두 개의 객체가 같은 경우 {@link java.lang.AssertionError AssertionError}를 발생함. 다른
	 * 메소드와의 일관성 유지를 위해 만듦
	 *
	 * @param object
	 *            예상되는 객체
	 * @param another
	 *            실제 결과 객체
	 * @see java.lang.AssertionError
	 * @see AssertJUnit#assertNotSame
	 */
	public void notSame(Object object, Object another) {

		AssertJUnit.assertNotSame(object, another);

	}

	/**
	 * 두 개의 객체가 다른 경우 {@link java.lang.AssertionError AssertionError}를 발생함. 다른
	 * 메소드와의 일관성 유지를 위해 만듦
	 *
	 * @param object
	 *            예상되는 객체
	 * @param another
	 *            실제 결과 객체
	 * @see java.lang.AssertionError
	 * @see AssertJUnit#assertSame
	 */
	public void same(Object object, Object another) {

		AssertJUnit.assertSame(object, another);

	}

	/**
	 * Spring 컨텍스트 파일에 dataSource 빈을 정의한 경우에 한해서
	 * SimpleSqlExecutor 객체를 제공
	 * @return
	 */
	public SimpleSqlExecutor getSqlExecutor() {

		SimpleSqlExecutor executor = new SimpleSqlExecutor();
		DataSource dataSource = (DataSource) getBean("dataSource");
		executor.setDataSource(dataSource);
		return executor;

	}

}
