package net.webapp2.common.managedobject.dao;

/**
 * 관리대상으로 지정된 객체의 데이터 저장을 담당
 *
 * @author younghoe
 *
 */
public interface ManagedObjectDao {

	/**
	 * 데이터 삭제에 따른 갱신
	 */
	public static final int DELETE = -1;

	/**
	 * 데이터 추가에 따른 갱신
	 */
	public static final int INSERT = 1;

	/**
	 * 기본 관리대상으로 지정된 객체의 개수 추정
	 *
	 * @return 기본 관리대상으로 지정된 객체의 개수 추정치.
	 * 기본 관리대상으로 어떠한 클래스나 레코드 유형도 지정하지 않은 경우 -1
	 */
	Integer count();

	/**
	 * 주어진 클래스의 관리대상 객체 개수 추정
	 *
	 * @param clazz
	 *            관리대상 클래스
	 * @return 개수 추정치. 주어진 클래스를 지원하지 않거나, 매개변수가 null인 경우 -1
	 */
	Integer count(Class clazz);

	/**
	 * 주어진 레코드 유형의 관리대상 객체 개수 추정
	 *
	 * @param recordType
	 *            레코드 유형
	 * @return 개수 추정치.
	 * 주어진  레코드 유형을 지원하지 않거나, 매개변수가 null인 경우 -1
	 */
	Integer count(String recordType);

	/**
	 * 기본 관리대상으로 지정된 객체의 삭제 회수 제공
	 *
	 * @return 기본 관리대상으로 지정된 객체의 삭제 회수.
	 * 기본 관리대상으로 어떠한 클래스나 레코드 유형도 지정하지 않은 경우 -1
	 */
	Integer howManyDeleted();

	/**
	 * 주어진 클래스의 관리대상 객체에 대한 삭제 회수 제공
	 *
	 * @param clazz
	 *            관리대상 클래스
	 * @return 삭제된 관리대상 개수.
	 * 주어진 클래스를 지원하지 않거나, 매개변수가 null인 경우 -1
	 */
	Integer howManyDeleted(Class clazz);

	/**
	 * 주어진 레코드 유형의 관리대상 객체에 대한 삭제 회수 제공
	 *
	 * @param recordType
	 *            레코드 유형
	 * @return 삭제된 관리대상 개수.
	 * 주어진  레코드 유형을 지원하지 않거나, 매개변수가 null인 경우 -1
	 */
	Integer howManyDeleted(String recordType);

	/**
	 * 기본 관리대상으로 지정된 객체의 관리번호 발행 횟수 제공
	 *
	 * @return 기본 관리대상의 관리번호 발행 횟수.
	 *  기본 관리대상으로 어떠한 클래스나 레코드 유형도 지정하지 않은 경우 -1
	 */
	Integer howManyIssued();

	/**
	 * 주어진 클래스의 관리대상 객체에 대한 관리번호 발행 횟수 제공
	 *
	 * @param clazz
	 *            관리대상 클래스
	 * @return 관리번호 발행 횟수.
	 * 주어진 클래스를 지원하지 않거나, 매개변수가 null인 경우 -1
	 */
	int howManyIssued(Class clazz);

	/**
	 * 주어진 레코드 유형의 관리대상 객체에 대한 관리번호 발행 횟수 제공
	 *
	 * @param recordType
	 *            레코드 유형
	 * @return 관리번호 발행 횟수.
	 * 주어진  레코드 유형을 지원하지 않거나, 매개변수가 null인 경우 -1
	 */
	int howManyIssued(String recordType);

	/**
	 * 기본 관리대상을 지정된 클래스 혹은 레코드 유형에 속하는 새로운 객체 생성을 위해
	 * 새 관리번호 발행
	 *
	 * @return 기본 관리대상에 대한 새 관리번호.
	 * 기본 관리대상으로 어떠한 클래스나 레코드 유형도 지정하지 않은 경우 -1
	 */
	Integer issue();

	/**
	 * 주어진 클래스의 새로운 관리대상 객체에 대한 관리번호 발행.
	 *
	 * @param clazz
	 *            관리대상 클래스
	 * @return 주어진 클래스에 대한 새 관리번호.
	 * 주어진 클래스를 지원하지 않거나, 매개변수가 null인 경우 -1
	 *
	 */
	Integer issue(Class clazz);

	/**
	 * 주어진 레코드 유형의 새로운 관리대상 객체에 대한 관리번호 발행.
	 *
	 * @param recordType
	 *            레코드 유형
	 * @return 주어진 레코드 유형의 관리대상에 대한 새 관리번호.
	 * 주어진  레코드 유형을 지원하지 않거나, 매개변수가 null인 경우 -1
	 */
	Integer issue(String recordType);

	/**
	 * 주어진 클래스의 객체에 대한 발행번호 및 수치 정보 관리를 지원하는지 여부
	 *
	 * @param clazz
	 *            클래스
	 * @return true - 지원하는 경우,
	 * false - 주어진 클래스를 지원하지 않거나, 매개변수가 null인 경우
	 */
	boolean supports(Class clazz);

	/**
	 * 주어진 문자열에 해당하는 레코드 유형의 객체에 대한
	 * 발행번호 및 수치 정보 관리를 지원하는지 여부
	 *
	 * @param recordType
	 *            레코드 유형
	 * @return true - 지원하는 경우,
	 * false - 주어진  레코드 유형을 지원하지 않거나, 매개변수가 null인 경우
	 */
	boolean supports(String recordType);

	/**
	 * 기본 관리대상으로 지정된 객체의 추가 및 삭제에 따른 데이터 변경.
	 * 갱신을 실패하는 경우 예외처리를 발생해야 함.
	 *
	 * @param updateType
	 *            1 - 추가에 따른 갱신, -1 - 삭제에 따른 갱신
	 */
	void updateFor(int updateType);

	/**
	 * 주어진 클래스의 관리대상 객체의 추가 및 삭제에 따른 데이터 변경.
	 * 갱신을 실패하는 경우 예외처리를 발생해야 함.
	 *
	 * @param updateType
	 *            1 - 추가에 따른 갱신, -1 - 삭제에 따른 갱신
	 * @param clazz
	 *            관리대상 클래스
	 */
	void updateFor(int updateType, Class clazz);

	/**
	 * 주어진 레코드 유형의 관리대상 객체의 추가 및 삭제에 따른 데이터 변경.
	 * 갱신을 실패하는 경우 예외처리를 발생해야 함.
	 * @param updateType
	 *            1 - 추가에 따른 갱신, -1 - 삭제에 따른 갱신
	 * @param recordType
	 *            관리대상 레코드 유형
	 */
	void updateFor(int updateType, String recordType);

}